package org.jeecg.modules.app.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import lombok.extern.log4j.Log4j2;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.app.dto.GetPsnAuthUrlDTO;
import org.jeecg.modules.app.esign.comm.EsignHttpResponse;
import org.jeecg.modules.app.esign.exception.EsignDemoException;
import org.jeecg.modules.app.esign.fileAndTemplate.FileDemo;
import org.jeecg.modules.app.esign.sign.SignDemo;
import org.jeecg.modules.app.service.IESignService;
import org.jeecg.modules.system.entity.SysUser;
import org.jeecg.modules.system.mapper.SysUserMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import static com.alibaba.dashscope.utils.JsonUtils.gson;
import static org.jeecg.modules.app.esign.auth.ESignAuth.*;
import static org.jeecg.modules.app.esign.fileAndTemplate.TemplateDemo.getTemplateCreateUrl;
import static org.jeecg.modules.app.esign.fileAndTemplate.TemplateDemo.getTemplateEditUrl;
import static org.jeecg.modules.app.esign.sign.SignDemo.createByFile;

@Service
@Log4j2
public class ESignServiceImpl implements IESignService {
    @Resource
    private SysUserMapper sysUserMapper;
    @Override
    public boolean getUserIdentityInfo(String phoneNumber) throws EsignDemoException {
        EsignHttpResponse getPsnIdentityInfo = getPsnIdentityInfo(phoneNumber);
        JsonObject getPsnIdentityInfoObject = gson.fromJson(getPsnIdentityInfo.getBody(), JsonObject.class);
        String realnameStatus =getPsnIdentityInfoObject.getAsJsonObject("data").get("realnameStatus").getAsString();//授权状态authorizeUserInfo
        return realnameStatus.equals("1");
    }

    @Override
    public String getAuthUrl() throws EsignDemoException {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        SysUser sysUser = sysUserMapper.selectById(user.getId());
        if (null == sysUser.getPhone()) throw new JeecgBootException("请先绑定手机号");
        GetPsnAuthUrlDTO getPsnAuthUrlDTO = new GetPsnAuthUrlDTO();
        GetPsnAuthUrlDTO.PsnAuthConfig pncAuthConfig = new GetPsnAuthUrlDTO.PsnAuthConfig().setPsnAccount(sysUser.getPhone());
        getPsnAuthUrlDTO.setPsnAuthConfig(pncAuthConfig);
        EsignHttpResponse getPsnAuthUrl = getPsnAuthUrl(getPsnAuthUrlDTO);
        JsonObject getPsnAuthUrlObject = gson.fromJson(getPsnAuthUrl.getBody(), JsonObject.class);
        JsonObject data = getPsnAuthUrlObject.getAsJsonObject("data");
        return data.get("authUrl").getAsString();
    }

    @Override
    public boolean getOrgIdentityInfos(String orgName) throws EsignDemoException {
        EsignHttpResponse getOrgIdentityInfo = getOrgIdentityInfo(orgName);
        JsonObject getOrgIdentityInfoObject = gson.fromJson(getOrgIdentityInfo.getBody(), JsonObject.class);
        String orgRealnameStatus =	getOrgIdentityInfoObject.getAsJsonObject("data").get("realnameStatus").getAsString();//授权状态authorizeUserInfo
        return orgRealnameStatus.equals("1");
    }

    @Override
    public String getOrgAuthUrls(String orgName,String phoneNumber) throws EsignDemoException {
        EsignHttpResponse getOrgAuthUrl = getOrgAuthUrl(orgName,phoneNumber);
        JsonObject getOrgAuthUrlObject = gson.fromJson(getOrgAuthUrl.getBody(), JsonObject.class);
        return getOrgAuthUrlObject.getAsJsonObject("data").get("authUrl").getAsString();
    }

    @Override
    public boolean uploadContractTemplate(String filePath) throws EsignDemoException {
        EsignHttpResponse getUploadUrl = FileDemo.getUploadUrl(filePath);
        JsonObject getUploadUrlJsonObject = gson.fromJson(getUploadUrl.getBody(), JsonObject.class);
        JsonObject data = getUploadUrlJsonObject.getAsJsonObject("data");
        String fileUploadUrl =data.get("fileUploadUrl").getAsString();
        EsignHttpResponse uploadFileResponse = FileDemo.uploadFile(fileUploadUrl,filePath);
        JsonObject uploadFileResponseJsonObject = gson.fromJson(uploadFileResponse.getBody(), JsonObject.class);
        int errCode = uploadFileResponseJsonObject.get("errCode").getAsInt();
        return errCode == 0;
    }

    @Override
    public String createSign(String fileId) throws EsignDemoException {
        EsignHttpResponse createByFile = createByFile();
        JSONObject createByFileJsonObject = JSONObject.parseObject(createByFile.getBody());
        String signFlowId = createByFileJsonObject.getJSONObject("data").getString("signFlowId");
        System.out.println("signFlowId:" + signFlowId);
        return "";
    }

    @Override
    public String getSignUrl(String signFlowId) throws EsignDemoException {
        JSONObject signUrl = JSONObject.parseObject(SignDemo.signUrl(signFlowId).getBody());
        return signUrl.getJSONObject("data").getString("url");
    }

    @Override
    public String createTemplate(String fileId) throws EsignDemoException {
        /* 获取制作合同模板页面*/
        EsignHttpResponse getTemplateCreateUrl = getTemplateCreateUrl(fileId);
        Gson gson = new Gson();
        JsonObject getTemplateCreateUrlObject = gson.fromJson(getTemplateCreateUrl.getBody(), JsonObject.class);

        String docTemplateCreateUrl = getTemplateCreateUrlObject.getAsJsonObject("data").get("docTemplateCreateUrl").getAsString();
        String docTemplateId = getTemplateCreateUrlObject.getAsJsonObject("data").get("docTemplateId").getAsString();
        System.err.println("制作合同模板的页面链接:"+docTemplateCreateUrl);
        System.err.println("合同模板id:"+docTemplateId);
        return "";
    }

    @Override
    public String editContract(String docTemplateId) throws EsignDemoException {
        /* 获取编辑合同模板页面*/
        EsignHttpResponse getTemplateEditUrl = getTemplateEditUrl();
        JsonObject getTemplateEditUrlObject = gson.fromJson(getTemplateEditUrl.getBody(), JsonObject.class);
        String docTemplateEditUrl = getTemplateEditUrlObject.getAsJsonObject("data").get("docTemplateEditUrl").getAsString();
        System.err.println("编辑合同模板的页面链接:"+docTemplateEditUrl);
        return "";
    }
}

