package org.jeecg.modules.app.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.app.service.IGameService;
import org.jeecg.modules.app.vo.game.*;
import org.jeecg.modules.system.app.mapper.AppGameMapper;
import org.jeecg.modules.system.app.mapper.AppGamePriceRulesMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Description: 赛事表
 * @Author: jeecg-boot
 * @Date: 2025-07-03
 * @Version: V1.0
 */
@Service
public class GameServiceImpl  implements IGameService {


    @Autowired
    private AppGameMapper appGameMapper;
    @Autowired
    private AppGamePriceRulesMapper appGamePriceRulesMapper;


    @Override
    public Result<IPage<FindPagResponse>> findConcerts(FindPageGameVO findPageGame) {

        if (StringUtils.isEmpty(findPageGame.getLatitude())||StringUtils.isEmpty(findPageGame.getLongitude())){
            return Result.error("经纬度不能为空");
        }
        double latitude = 0;
        double longitude = 0;
        try {
            latitude = Double.parseDouble(findPageGame.getLatitude());
            longitude = Double.parseDouble(findPageGame.getLongitude());
        } catch (NumberFormatException e) {
            return Result.error("经纬度格式错误");
        }
        Page<FindPagResponse> page = new Page<>(findPageGame.getPageNo(), findPageGame.getPageSize());

        return Result.OK(appGameMapper.findConcerts(page,longitude,latitude)) ;
    }

    @Override
    public Result<FindByIdResponse> findById(String id,String longitude, String latitude) {
        if (StringUtils.isEmpty(id)){
            return Result.error("未选择赛事");
        }
        if (StringUtils.isEmpty(longitude)||StringUtils.isEmpty(latitude)){
            return Result.error("经纬度不能为空");
        }
        double latitudeNew = 0;
        double longitudeNew = 0;
        try {
            latitudeNew = Double.parseDouble(latitude);
            longitudeNew = Double.parseDouble(longitude);
        } catch (NumberFormatException e) {
            return Result.error("经纬度格式错误");
        }
        FindByIdResponse findByIdResponse =  appGameMapper.findById(id,longitudeNew,latitudeNew);
        if (findByIdResponse!=null&&StringUtils.isNotEmpty(findByIdResponse.getBackgroundImage())){
            List<String> backgroundImageList = new ArrayList<>();
            if (findByIdResponse.getBackgroundImage().indexOf(",")>0){
                backgroundImageList= Arrays.asList(findByIdResponse.getBackgroundImage().split(","));
            }else {
                backgroundImageList.add(findByIdResponse.getBackgroundImage());
            }
            findByIdResponse.setBackgroundImageList(backgroundImageList);
        }
        return Result.OK(findByIdResponse);
    }

    @Override
    public Result<FindByGameIdPriceVo> findByGameId(String id) {
        FindByGameIdPriceVo findByGameIdPriceVo = appGameMapper.findByGameId(id);
        if (findByGameIdPriceVo==null){
            return Result.error("该赛事未查询到比赛");
        }
        if (StringUtils.isNotEmpty(findByGameIdPriceVo.getAptitudes())){
            List<String> aptitudesList = new ArrayList<>();
            if (findByGameIdPriceVo.getAptitudes().indexOf(",")>0){
                aptitudesList =  Arrays.asList(findByGameIdPriceVo.getAptitudes().split(","));
            }else {
                aptitudesList.add(findByGameIdPriceVo.getAptitudes());
            }
            findByGameIdPriceVo.setAptitudesList(aptitudesList);
        }
        if (ObjectUtils.isNotEmpty(findByGameIdPriceVo.getGamePriceRulesTypeVOList())&&!findByGameIdPriceVo.getGamePriceRulesTypeVOList().isEmpty()){
            for (GamePriceRulesTypeVO gamePriceRulesTypeVO : findByGameIdPriceVo.getGamePriceRulesTypeVOList()) {
                if (ObjectUtils.isNotEmpty(gamePriceRulesTypeVO.getType())){
                    List<GamePriceRulesVO> gamePriceRulesVOList=  appGamePriceRulesMapper.findByGameIdList(id,gamePriceRulesTypeVO.getType());
                    gamePriceRulesTypeVO.setGamePriceRulesVOList(gamePriceRulesVOList);
                }
            }
        }
        return Result.OK(findByGameIdPriceVo);
    }

}
