package org.jeecg.modules.system.app.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.system.app.dto.*;
import org.jeecg.modules.system.app.entity.AppCategory;
import org.jeecg.modules.system.app.entity.AppCourses;
import org.jeecg.modules.system.app.entity.AppCoursesPriceRules;
import org.jeecg.modules.system.app.entity.AppSite;
import org.jeecg.modules.system.app.mapper.AppCategoryMapper;
import org.jeecg.modules.system.app.mapper.AppCoursesMapper;
import org.jeecg.modules.system.app.mapper.AppCoursesPriceRulesMapper;
import org.jeecg.modules.system.app.mapper.AppSiteMapper;
import org.jeecg.modules.system.app.service.IAppCoureseService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static org.jeecg.common.constant.CommonConstant.SC_INTERNAL_SERVER_ERROR_500;

/**
 * @Description: 课程/培训表
 * @Author: jeecg-boot
 * @Date: 2025-07-03
 * @Version: V1.0
 */
@Service
public class AppCoureseServiceImpl extends ServiceImpl<AppCoursesMapper, AppCourses> implements IAppCoureseService {
    @Resource
    private AppCoursesPriceRulesMapper priceRulesMapper;
    @Resource
    private AppSiteMapper siteMapper;
    @Resource
    private AppCategoryMapper categoryMapper;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean saveWitchPriceRules(AppCoursesDTO appCoursesDTO) {
        AppCoursesCuDTO coursesCuDTO = appCoursesDTO.getCourses();
        if (null == coursesCuDTO.getCategoryId()) throw new JeecgBootException("请选择课程类别");
        AppSite site = siteMapper.selectOne(Wrappers.<AppSite>lambdaQuery().eq(AppSite::getTenantId, coursesCuDTO.getSiteId()));
        AppCourses appCourses = new AppCourses();
        LoginUser loginUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        BeanUtils.copyProperties(coursesCuDTO, appCourses);
        appCourses.setOrgCode(site.getOrgCode());
        appCourses.setTenantId(site.getTenantId());
        appCourses.setUserId(coursesCuDTO.getUserId());
        appCourses.setAddressSiteId(coursesCuDTO.getAddressSiteId());
        List<AppCoursesRuleDTO> coursesRuleDTOS= appCoursesDTO.getPriceRulesList();
        //起止时间计算
        Optional<Date> minStartTime = coursesRuleDTOS.stream()
                .map(AppCoursesRuleDTO::getStartTime)
                .filter(Objects::nonNull)
                .min(Date::compareTo);

        Optional<Date> maxEndTime = coursesRuleDTOS.stream()
                .map(AppCoursesRuleDTO::getEndTime)
                .filter(Objects::nonNull)
                .max(Date::compareTo);
        appCourses.setStartTime(minStartTime.orElse(null));
        appCourses.setEndTime(maxEndTime.orElse(null));
        int saveCourseResult = baseMapper.insert(appCourses);
        if (saveCourseResult < 1) {
            throw new JeecgBootException("课程信息保存失败");
        }
        for (AppCoursesRuleDTO rule : coursesRuleDTOS) {
            AppCoursesPriceRules appCoursesPriceRules = new AppCoursesPriceRules();
            BeanUtils.copyProperties(rule, appCoursesPriceRules);
            appCoursesPriceRules.setCoursesId(appCourses.getId());
            appCoursesPriceRules.setOrgCode(appCourses.getOrgCode());
            appCoursesPriceRules.setTenantId(appCourses.getTenantId());
            int savePriceResult = priceRulesMapper.insert(appCoursesPriceRules); // 保证每个操作都在事务中
            if (savePriceResult < 1) {
                throw new JeecgBootException("课程价格规则保存失败");
            }
        }
        return Boolean.TRUE;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean editWitchPriceRules(AppCoursesDTO appCoursesDTO) {
        AppCoursesCuDTO coursesCuDTO = appCoursesDTO.getCourses();
        if (null == coursesCuDTO.getCategoryId()) throw new JeecgBootException("请选择课程类别");

        AppCourses appCourses = baseMapper.selectById(coursesCuDTO.getId());
        BeanUtils.copyProperties(coursesCuDTO, appCourses);
        appCourses.setAddressSiteId(coursesCuDTO.getAddressSiteId());
        List<AppCoursesRuleDTO> coursesRuleDTOS= appCoursesDTO.getPriceRulesList();
        //起止时间计算
        Optional<Date> minStartTime = coursesRuleDTOS.stream()
                .map(AppCoursesRuleDTO::getStartTime)
                .filter(Objects::nonNull)
                .min(Date::compareTo);

        Optional<Date> maxEndTime = coursesRuleDTOS.stream()
                .map(AppCoursesRuleDTO::getEndTime)
                .filter(Objects::nonNull)
                .max(Date::compareTo);
        appCourses.setStartTime(minStartTime.orElse(null));
        appCourses.setEndTime(maxEndTime.orElse(null));
        int updateResult = baseMapper.updateById(appCourses);
        if (updateResult < 1) {
            throw new JeecgBootException("课程信息保存失败");
        }

        //查历史本地数据
        List<AppCoursesPriceRules> coursesPriceRules = priceRulesMapper.selectList(Wrappers.lambdaQuery(AppCoursesPriceRules.class).eq(AppCoursesPriceRules::getCoursesId, appCourses.getId()));
        List<String> localList = coursesPriceRules.stream().map(AppCoursesPriceRules::getCoursesId).collect(Collectors.toList());
        //表单数据
        List<String> formList = coursesRuleDTOS.stream().filter(e->ObjectUtils.isNotEmpty(e.getId())).map(AppCoursesRuleDTO::getId).collect(Collectors.toList());
        //有ID的做更新
        coursesRuleDTOS.stream().filter(e->ObjectUtils.isNotEmpty(e.getId())).forEach(coursesRule -> {
            AppCoursesPriceRules appCoursesPriceRules = new AppCoursesPriceRules();
            BeanUtils.copyProperties(coursesRule, appCoursesPriceRules);
            priceRulesMapper.updateById(appCoursesPriceRules);
        });
        //差集直接做删除
        List<String> differ = getDifferListByMap(formList, localList);
        //修改状态
        for (String id : differ) {
            AppCoursesPriceRules appCoursesPriceRules = priceRulesMapper.selectById(id);
            if (ObjectUtils.isNotEmpty(appCoursesPriceRules)) {
                appCoursesPriceRules.setStatus(CommonConstant.STATUS_1_INT);
                priceRulesMapper.updateById(appCoursesPriceRules);
            }
        }
        //id为空做新增
        List<AppCoursesRuleDTO> nullList = coursesRuleDTOS.stream().filter(e->ObjectUtils.isEmpty(e.getId())).collect(Collectors.toList());
        for (AppCoursesRuleDTO rule : nullList) {
            //做新增
            AppCoursesPriceRules appCoursesPriceRules = new AppCoursesPriceRules();
            BeanUtils.copyProperties(rule, appCoursesPriceRules);
            appCoursesPriceRules.setCoursesId(appCourses.getId());
            appCoursesPriceRules.setOrgCode(appCourses.getOrgCode());
            appCoursesPriceRules.setTenantId(appCourses.getTenantId());
            int savePriceResult = priceRulesMapper.insert(appCoursesPriceRules); // 保证每个操作都在事务中
            if (savePriceResult < 1) {
                throw new JeecgBootException("课程价格规则保存失败");
            }
        }
        return Boolean.TRUE;
    }


    public static List<String> getDifferListByMap(List<String> listA, List<String> listB) {
        List<String> differList = new ArrayList<>();
        Map<String, Integer> map = new HashMap<>();
        for (String strA : listA) {
            map.put(strA, 1);
        }
        for (String strB : listB) {
            Integer value = map.get(strB);
            if (value != null) {
                map.put(strB, ++value);
                continue;
            }
            map.put(strB, 1);
        }
        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            if (entry.getValue() == 1) {
                differList.add(entry.getKey());
            }
        }
        return differList;
    }

    @Override
    public AppCoursesDTO queryWitchPriceRulesById(String id) {
        AppCourses dbCourse = baseMapper.selectById(id);
        if (null==dbCourse) throw new JeecgBootException("未找到对应数据", SC_INTERNAL_SERVER_ERROR_500);
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
//        checkPermission(sysUser,dbCourse);
        List<AppCoursesPriceRules> priceRulesList =
                priceRulesMapper.selectList(Wrappers.<AppCoursesPriceRules>lambdaQuery().eq(AppCoursesPriceRules::getCoursesId, id));
        AppCoursesCuDTO coursesCuDTO = new AppCoursesCuDTO();
        BeanUtils.copyProperties(dbCourse, coursesCuDTO);
        List<AppCoursesRuleDTO> priceRules =new ArrayList<>();
        priceRulesList.forEach(rule-> {
            AppCoursesRuleDTO appCoursesRuleDTO = new AppCoursesRuleDTO();
            BeanUtils.copyProperties(rule, appCoursesRuleDTO);
            priceRules.add(appCoursesRuleDTO);
        });

        return new AppCoursesDTO(coursesCuDTO, priceRules);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteWitchPriceRulesById(String id) {
        AppCourses dbCourse = baseMapper.selectById(id);
        if (null==dbCourse) throw new JeecgBootException("未找到对应数据", SC_INTERNAL_SERVER_ERROR_500);
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
//        checkPermission(sysUser,dbCourse);
        int deleteCourseResult = baseMapper.deleteById(id);
        if (deleteCourseResult < 1) {
            throw new JeecgBootException("课程信息删除失败", SC_INTERNAL_SERVER_ERROR_500);
        }
        int deletePriceResult = priceRulesMapper.delete(Wrappers.<AppCoursesPriceRules>lambdaQuery().eq(AppCoursesPriceRules::getCoursesId, id));
        if (deletePriceResult<1) throw new JeecgBootException("课程价格规则删除失败", SC_INTERNAL_SERVER_ERROR_500);
        return Boolean.TRUE;
    }

    @Override
    public IPage<AppCoursesInfoPageDTO> queryWitchPriceRulesPage(AppCoursesPageDTO dto) {
        LoginUser loginUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        dto.setOrgCode(loginUser.getOrgCode());
        Page<AppCourses> page = new Page<>(dto.getPageNum(), dto.getPageSize());
        IPage<AppCoursesInfoPageDTO> resultPage = baseMapper.queryWitchPriceRulesPage(page, dto);

        resultPage.getRecords().forEach(r -> {
            String[] split = r.getCategoryId().split(",");
            List<String> ids = Arrays.stream(split).collect(Collectors.toList());

            Long count = priceRulesMapper.selectCount(Wrappers.<AppCoursesPriceRules>lambdaQuery().eq(AppCoursesPriceRules::getCoursesId, r.getId()));

            r.setTotalNum(Math.toIntExact(count));

            StringJoiner stringJoiner = new StringJoiner(",");

            for (String id : ids) {
                AppCategory category = categoryMapper.selectById(id);
                stringJoiner.add(category.getName());
            }
            r.setCategoryIds(stringJoiner.toString());
        });
        return resultPage;
    }

    @Override
    public List<AppSiteAddressDTO> queryAddressList() {
        List<AppSite> appSites = siteMapper.selectList(Wrappers.<AppSite>lambdaQuery().select(AppSite::getId, AppSite::getAddress));
        List<AppSiteAddressDTO> dtoList = Optional.ofNullable(appSites.stream().map(e -> {
            AppSiteAddressDTO appSiteAddressDTO = new AppSiteAddressDTO();
            appSiteAddressDTO.setSiteId(e.getId());
            appSiteAddressDTO.setAddress(e.getAddress());
            return appSiteAddressDTO;
        }).collect(Collectors.toList())).orElse(List.of());
        return dtoList;
    }

    /**
     *  校验权限
     * @param loginUser
     * @param course
     */
    private void checkPermission(LoginUser loginUser, AppCourses course) {
        if (!loginUser.getOrgCode().equals(course.getOrgCode())) throw new JeecgBootException("无权限操作", SC_INTERNAL_SERVER_ERROR_500);
    }
}
